from django import forms
from django.utils.datastructures import MultiValueDict


class MultipleFileInput(forms.ClearableFileInput):
    def render(self, name, value, attrs=None, renderer=None):
        attrs["multiple"] = "multiple"

        return super(MultipleFileInput, self).render(name, value, attrs)

    def value_from_datadict(self, data, files, name):
        if isinstance(files, MultiValueDict):
            return files.getlist(name)

        return []



from django.conf import settings
from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.contrib.auth.views import redirect_to_login
from django.db import transaction
from django.http.response import HttpResponseRedirect
from django.shortcuts import render
from django.urls import path, reverse
from django.views.decorators.http import require_http_methods

from django_support_lite.enums import Priority, ResponseType, Status
from django_support_lite.helpers import email
from django_support_lite.helpers.pagination import InvalidPageError, Pagination
from django_support_lite.helpers.upload import upload
from django_support_lite.models.image import Image
from django_support_lite.models.message import Message
from django_support_lite.models.ticket import Ticket
from django_support_lite.views.forms.message_create_form import MessageCreateForm
from django_support_lite.views.forms.ticket_create_form import TicketCreateForm


@login_required
@permission_required('django_support_lite.can_access')
@require_http_methods(['GET', 'POST'])
def view(request, ticket_id):
    try:
        ticket = Ticket.objects.select_related('user').get(pk=ticket_id)
    except Ticket.DoesNotExist:
        return HttpResponseRedirect(
            reverse('django_support_lite:tickets.list')
        )

    if not request.user.has_perm('django_support_lite.can_manage'):
        if ticket.user.id != request.user.id:
            return redirect_to_login(
                request.get_full_path()
            )

    if request.method == 'POST':
        form = MessageCreateForm(request.POST, request.FILES)

        if form.is_valid():
            with transaction.atomic():
                message = Message.objects.create(
                    ticket=ticket,
                    user=request.user,
                    text=form.cleaned_data['text']
                )

                # TODO: delete image on model delete
                for file in form.cleaned_data['images']:
                    Image.objects.create(
                        message=message,
                        path=upload(file)
                    )

                if request.user.has_perm('django_support_lite.can_manage'):
                    ticket.response_type = ResponseType.SUPPORT_RESPONSE
                else:
                    ticket.response_type = ResponseType.USER_RESPONSE

                ticket.save()

            def send_message_created(to):
                nonlocal request, ticket, message
                email.send(
                    request,
                    to,
                    'Response in ticket',
                    'message_created',
                    {
                        'url': request.build_absolute_uri(
                            reverse(
                                'django_support_lite:tickets.view',
                                kwargs={'ticket_id': ticket.id}
                            )
                        ) + '#message-{}'.format(message.id)
                    }
                )

            if request.user.has_perm('django_support_lite.can_manage'):
                if ticket.user.email:
                    send_message_created(ticket.user.email)
            else:
                if hasattr(settings, 'DSL_EMAIL_MANAGER'):
                    send_message_created(settings.DSL_EMAIL_MANAGER)

            messages.success(request, 'Message created.')

            return HttpResponseRedirect(
                reverse('django_support_lite:tickets.view', kwargs={'ticket_id': ticket_id})
            )

        messages.error(request, 'Error creating ticket message.')
    else:
        form = MessageCreateForm(initial=ticket.__dict__)

    return render(
        request,
        'views/tickets/view.html',
        {
            'title': 'View Ticket',
            'ticket': ticket,
            'ticket_messages': ticket.message_set.select_related('user').prefetch_related('image_set').order_by('-created_at').all(),
            'form': form,
            'Priority': Priority,
            'ResponseType': ResponseType,
            'Status': Status,
            'DSL_LAYOUT_NAME': settings.DSL_LAYOUT_NAME,
            'DSL_UPLOAD_URL_PREFIX': settings.DSL_UPLOAD_URL_PREFIX,
        }
    )

@login_required
@permission_required('django_support_lite.can_access')
@require_http_methods(['GET', 'POST'])
def create(request):
    if request.user.has_perm('django_support_lite.can_manage'):
        return HttpResponseRedirect(
            reverse('django_support_lite:tickets.list')
        )

    if request.method == 'POST':
        form = TicketCreateForm(request.POST, request.FILES)

        if form.is_valid():
            with transaction.atomic():
                ticket = Ticket.objects.create(
                    user=request.user,
                    title=form.cleaned_data['title'],
                    response_type=ResponseType.USER_RESPONSE,
                    status=Status.OPEN,
                    priority=form.cleaned_data['priority']
                )

                message = Message.objects.create(
                    ticket=ticket,
                    user=request.user,
                    text=form.cleaned_data['text']
                )

                # TODO: delete image on model delete
                for file in form.cleaned_data['images']:
                    Image.objects.create(
                        message=message,
                        path=upload(file)
                    )

            if request.user.email:
                email.send(
                    request,
                    request.user.email,
                    'You\'ve created ticket',
                    'ticket_created_accessor',
                    {
                        'url': request.build_absolute_uri(
                            reverse(
                                'django_support_lite:tickets.view',
                                kwargs={'ticket_id': ticket.id}
                            )
                        ),
                    }
                )

            if hasattr(settings, 'DSL_EMAIL_MANAGER'):
                email.send(
                    request,
                    settings.DSL_EMAIL_MANAGER,
                    'User created ticket',
                    'ticket_created_manager',
                    {
                        'url': request.build_absolute_uri(
                            reverse(
                                'django_support_lite:tickets.view',
                                kwargs={'ticket_id': ticket.id}
                            )
                        ),
                    }
                )

            messages.success(request, 'Ticket created.')

            return HttpResponseRedirect(
                reverse('django_support_lite:tickets.view', kwargs={'ticket_id': ticket.id})
            )

        messages.error(request, 'Error creating the ticket.')
    else:
        form = TicketCreateForm()

    return render(
        request,
        'views/tickets/create.html',
        {
            'title': 'Create New Ticket',
            'form': form,
            'DSL_LAYOUT_NAME': settings.DSL_LAYOUT_NAME,
        }
    )

@login_required
@permission_required('django_support_lite.can_access')
@require_http_methods(['GET'])
def list_(request, page=1):
    try:
        pagination = Pagination(
            Ticket.objects.filter().count(),
            page,
            settings.DSL_TICKETS_PER_PAGE,
            'django_support_lite:tickets.list'
        )
    except InvalidPageError:
        return HttpResponseRedirect(
            reverse('django_support_lite:tickets.list')
        )

    tickets_query_set = Ticket.objects.select_related(
        'user'
    )

    if request.user.has_perm('django_support_lite.can_manage'):
        tickets_query_set = tickets_query_set.order_by(
            'status',
            'response_type',
            '-priority',
            '-updated_at'
        )
        count_pending = Ticket.objects.count_pending()
    else:
        tickets_query_set = tickets_query_set.order_by(
            'status',
            '-response_type',
            '-priority',
            '-updated_at'
        ).filter(user=request.user)
        count_pending = None

    return render(
        request,
        'views/tickets/list.html',
        {
            'title': 'Tickets List',
            'tickets': tickets_query_set.all()[pagination.slice],
            'count_pending': count_pending,
            'pagination': pagination,
            'Priority': Priority,
            'ResponseType': ResponseType,
            'Status': Status,
            'DSL_LAYOUT_NAME': settings.DSL_LAYOUT_NAME,
        }
    )

@login_required
@permission_required('django_support_lite.can_access')
@require_http_methods(['GET'])
def set_status(request, ticket_id, status):
    try:
        ticket = Ticket.objects.select_related('user').get(pk=ticket_id)
    except Ticket.DoesNotExist:
        return HttpResponseRedirect(
            reverse('django_support_lite:tickets.list')
        )

    if not request.user.has_perm('django_support_lite.can_manage'):
        if ticket.user.id != request.user.id:
            return redirect_to_login(
                request.get_full_path()
            )

        if ticket.status == Status.ARCHIVE:
            return redirect_to_login(
                request.get_full_path()
            )

        if status == Status.ARCHIVE:
            return redirect_to_login(
                request.get_full_path()
            )

    ticket.status = status

    if status == Status.CLOSE:
        ticket.user_close = request.user

    ticket.save()

    if 'HTTP_REFERER' not in request.META:
        return HttpResponseRedirect(
            reverse('django_support_lite:tickets.list')
        )

    return HttpResponseRedirect(
        request.META['HTTP_REFERER']
    )

@login_required
@permission_required('django_support_lite.can_manage')
@require_http_methods(['GET'])
def set_priority(request, ticket_id, priority):
    try:
        ticket = Ticket.objects.get(pk=ticket_id)
    except Ticket.DoesNotExist:
        return HttpResponseRedirect(
            reverse('django_support_lite:tickets.list')
        )

    ticket.priority = priority
    ticket.save()

    if 'HTTP_REFERER' not in request.META:
        return HttpResponseRedirect(
            reverse('django_support_lite:tickets.list')
        )

    return HttpResponseRedirect(
        request.META['HTTP_REFERER']
    )

urlpatterns = [
    path(
        'tickets/view/<int:ticket_id>',
        view, name='tickets.view'
    ),
    path(
        'tickets/create',
        create, name='tickets.create'
    ),
    path(
        'tickets/list',
        list_, name='tickets.list'
    ),
    path(
        'tickets/list/<int:page>',
        list_, name='tickets.list'
    ),
    path(
        'tickets/set-status-open/<int:ticket_id>',
        set_status, name='tickets.set_status_open', kwargs={'status': Status.OPEN}
    ),
    path(
        'tickets/set-status-close/<int:ticket_id>',
        set_status, name='tickets.set_status_close', kwargs={'status': Status.CLOSE}
    ),
    path(
        'tickets/set-status-archive/<int:ticket_id>',
        set_status, name='tickets.set_status_archive', kwargs={'status': Status.ARCHIVE}
    ),
    path(
        'tickets/set-priority/<int:ticket_id>/<int:priority>',
        set_priority, name='tickets.set_priority'
    ),
]



from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.template.loader import render_to_string


def send(request, to, title, template, context):
    if not hasattr(settings, 'DSL_EMAIL_ENABLE') or not settings.DSL_EMAIL_ENABLE:
        return

    email = EmailMultiAlternatives(
        title,
        render_to_string(
            'emails/{}.txt'.format(template),
            context,
            request
        ),
        settings.DSL_EMAIL_SENDER,
        [to]
    )
    email.attach_alternative(
        render_to_string(
            'emails/{}.html'.format(template),
            context,
            request
        ),
        "text/html"
    )
    email.send()



from django.template import Library

from django_support_lite.enums import Priority, ResponseType, Status

register = Library()

@register.filter
def ticket_state_label(ticket):
    if ticket.status == Status.OPEN:
        return ResponseType.label(ticket.response_type)

    return Status.label(ticket.status)

@register.filter
def ticket_state_css_class(ticket):
    if ticket.status in (Status.CLOSE, Status.ARCHIVE):
        return 'light'

    if ticket.response_type == ResponseType.USER_RESPONSE:
        return 'primary'

    if ticket.response_type == ResponseType.SUPPORT_RESPONSE:
        return 'secondary'

    raise ValueError('Response type not supported yet.')

@register.filter
def ticket_priority_label(priority):
    return Priority.label(priority)

@register.filter
def ticket_priority_css_class(priority):
    if priority == Priority.LOW:
        return 'success'

    if priority == Priority.NORMAL:
        return 'warning'

    if priority == Priority.HIGH:
        return 'danger'

    raise ValueError('Priority not supported yet.')



class Enum:
    OPTIONS = None

    @classmethod
    def label(cls, value):
        return cls._members()[value]

    @classmethod
    def choices(cls):
        return cls.OPTIONS

    # private

    members = None

    @classmethod
    def _members(cls):
        if not cls.members:
            cls.members = dict(cls.OPTIONS)

        return cls.members

class ResponseType(Enum):
    USER_RESPONSE = 0
    SUPPORT_RESPONSE = 1

    OPTIONS = [
        (USER_RESPONSE, 'Awaiting response'),
        (SUPPORT_RESPONSE, 'Support responded'),
    ]

class Status(Enum):
    OPEN = 0
    CLOSE = 1
    ARCHIVE = 2

    OPTIONS = [
        (OPEN, 'Open'),
        (CLOSE, 'Closed'),
        (ARCHIVE, 'Archived'),
    ]

class Priority(Enum):
    LOW = 0
    NORMAL = 1
    HIGH = 2

    OPTIONS = [
        (LOW, 'Low'),
        (NORMAL, 'Normal'),
        (HIGH, 'High'),
    ]



import uuid

from django.conf import settings


def upload(file):
    # TODO: add slashes, create directory to bypass files count limitation.
    suffix = str(uuid.uuid4())

    parts = file.name.split(".")
    if len(parts) >= 2:
        extension = '.{}'.format(parts[-1])
    else:
        extension = ''

    path = "{}/{}{}".format(
        settings.DSL_UPLOAD_DIRECTORY,
        suffix,
        extension
    )

    with open(path, "wb") as handle:
        handle.write(
            b"".join([chunk for chunk in file.chunks()])
        )

    return "{}{}".format(suffix, extension)



from django.db import models

from django_support_lite.models.message import Message


class Image(models.Model):
    id = models.AutoField(primary_key=True)
    message = models.ForeignKey(Message, on_delete=models.CASCADE)
    path = models.CharField(max_length=1024)



# Generated by Django 2.1.7 on 2019-02-15 06:49

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Image',
            fields=[
                ('id', models.AutoField(primary_key=True, serialize=False)),
                ('path', models.CharField(max_length=1024)),
            ],
        ),
        migrations.CreateModel(
            name='Message',
            fields=[
                ('id', models.AutoField(primary_key=True, serialize=False)),
                ('text', models.TextField(max_length=100000)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
            ],
        ),
        migrations.CreateModel(
            name='Ticket',
            fields=[
                ('id', models.AutoField(primary_key=True, serialize=False)),
                ('title', models.CharField(max_length=300)),
                ('response_type', models.IntegerField(choices=[(0, 'Awaiting response'), (1, 'Support responded')])),
                ('status', models.IntegerField(choices=[(0, 'Open'), (1, 'Closed'), (2, 'Archived')])),
                ('priority', models.IntegerField(choices=[(0, 'Normal priority'), (1, 'Moderate priority'), (2, 'High priority')])),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='user', to=settings.AUTH_USER_MODEL)),
                ('user_close', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='user_close', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AddField(
            model_name='message',
            name='ticket',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_support_lite.Ticket'),
        ),
        migrations.AddField(
            model_name='message',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='image',
            name='message',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_support_lite.Message'),
        ),
    ]



import os

import magic
from django.forms import ValidationError
from django.template.defaultfilters import filesizeformat


class FileContentValidator:
    def __init__(self, min_size=None, max_size=None, content_restrictions=[]):
        self.min_size = min_size
        self.max_size = max_size
        self.content_restrictions = content_restrictions

    def __call__(self, value):
        if self.min_size and value.size < self.min_size:
            raise ValidationError(
                "Размер файла должен быть больше {}. Текущий размер {}.".format(
                    filesizeformat(self.min_size),
                    filesizeformat(value.size)
                )
            )

        if self.max_size and value.size > self.max_size:
            raise ValidationError(
                "Размер файла должен быть меньше {}. Текущий размер {}.".format(
                    filesizeformat(self.max_size),
                    filesizeformat(value.size)
                )
            )

        if not self.content_restrictions:
            return

        extension = os.path.splitext(value.name)
        if len(extension) != 2:
            raise ValidationError("Файлы данного типа не поддерживаются.")

        extension = extension[1].replace(".", "").lower()
        content_type = magic.from_buffer(value.read(), mime=True)
        success_flag = False

        for content_restrictions in self.content_restrictions:
            if extension not in content_restrictions["extensions"]:
                continue

            if content_type not in content_restrictions["content_types"]:
                continue

            success_flag = True
            break

        if not success_flag:
            raise ValidationError("Файлы данного типа не поддерживаются.")



from django import forms

from django_support_lite.enums import Priority
from django_support_lite.forms.fields.multiple_file_field import MultipleFileField
from django_support_lite.forms.inputs.multiple_file_input import MultipleFileInput
from django_support_lite.forms.validators.file_content_validator import FileContentValidator


# TODO: use ModelForm
class TicketCreateForm(forms.Form):
    title = forms.CharField(
        widget=forms.TextInput({
            'class': 'form-control',
        }),
        label='Title',
        help_text='Ticket subject.',
        max_length=300
    )

    text = forms.CharField(
        widget=forms.Textarea({
            'class': 'form-control',
            'rows': 5,
        }),
        label='Text',
        help_text='Ticket message.',
        max_length=100000
    )

    priority = forms.ChoiceField(
        widget=forms.Select({
            'class': 'form-control',
        }),
        label='Priority',
        initial=Priority.NORMAL,
        choices=(
            (priority, Priority.label(priority))
            for priority in (
                Priority.LOW,
                Priority.NORMAL,
                Priority.HIGH,
            )
        )
    )

    images = MultipleFileField(
        widget=MultipleFileInput(),
        validators=[
            FileContentValidator(content_restrictions=[
                {
                    'extensions': [
                        'jpg',
                        'jpeg',
                        'gif',
                        'png',
                    ],
                    'content_types': [
                        'image/jpeg',
                        'image/jpeg',
                        'image/pjpeg',
                        'image/png',
                        'image/gif',
                    ],
                },
            ]),
        ],
        label='Images',
        help_text='Up to 5 images. Format jpg, png, gif.',
        max_count=5,
        required=False
    )



from django.contrib.auth import get_user_model as user_model
from django.db import models

from django_support_lite.models.ticket import Ticket


class Message(models.Model):
    id = models.AutoField(primary_key=True)
    ticket = models.ForeignKey(Ticket, on_delete=models.CASCADE)
    user = models.ForeignKey(user_model(), on_delete=models.CASCADE)
    text = models.TextField(max_length=100000)
    created_at = models.DateTimeField(auto_now_add=True)



import os

from django.conf import settings
from django.core import checks


@checks.register()
def check_settings(app_configs=None, **kwargs):
    errors = [
        checks.Error(
            'Setting {} shoud be set.'.format(setting)
        )
        for setting in [
            'DSL_LAYOUT_NAME',
            'DSL_UPLOAD_DIRECTORY',
            'DSL_UPLOAD_URL_PREFIX',
            'DSL_TICKETS_PER_PAGE',
            'DSL_EMAIL_ENABLE',
        ]
        if not hasattr(settings, setting)
    ]

    if hasattr(settings, 'DSL_EMAIL_ENABLE') and settings.DSL_EMAIL_ENABLE:
        if not hasattr(settings, 'DSL_EMAIL_SENDER'):
            errors.append(
                checks.Error(
                    'Setting {} shoud be set.'.format('DSL_EMAIL_SENDER')
                )
            )

        if not hasattr(settings, 'DSL_EMAIL_MANAGER'):
            errors.append(
                checks.Error(
                    'Setting {} shoud be set.'.format('DSL_EMAIL_MANAGER')
                )
            )

    return errors

@checks.register()
def check_upload_directory(app_configs=None, **kwargs):
    if os.path.exists(settings.DSL_UPLOAD_DIRECTORY):
        return []

    return [
        checks.Error('Upload directory not exist.'),
    ]



from django import forms

from django_support_lite.forms.fields.multiple_file_field import MultipleFileField
from django_support_lite.forms.inputs.multiple_file_input import MultipleFileInput
from django_support_lite.forms.validators.file_content_validator import FileContentValidator


# TODO: use ModelForm
class MessageCreateForm(forms.Form):
    text = forms.CharField(
        widget=forms.Textarea({
            "class": "form-control",
            "rows": 5,
        }),
        label="Text",
        help_text="Ticket message.",
        max_length=100000
    )

    images = MultipleFileField(
        widget=MultipleFileInput(),
        validators=[
            FileContentValidator(content_restrictions=[
                {
                    "extensions": [
                        "jpg",
                        "jpeg",
                        "gif",
                        "png",
                    ],
                    "content_types": [
                        "image/jpeg",
                        "image/jpeg",
                        "image/pjpeg",
                        "image/png",
                        "image/gif",
                    ],
                },
            ]),
        ],
        label="Images",
        help_text="Up to 5 images. Format jpg, png, gif.",
        max_count=5,
        required=False
    )



from django.template import Library

register = Library()


@register.filter
def form_field_class(field):
    return field.field.widget.__class__.__name__

@register.filter
def form_field_group(field, groups):
    if field not in groups:
        return None

    return groups[field]



import math
from functools import partial
from urllib.parse import urlencode

from django.urls import reverse


class InvalidPageError(Exception):
    pass

class Pagination:
    def __init__(self, count, page, per_page, name, path_params={}, query_params={}, fragment=None):
        if page < 1:
            raise InvalidPageError()

        pages_count = math.ceil(count / per_page)

        if pages_count == 0:
            pages_count = 1

        if page > pages_count:
            raise InvalidPageError()

        url = partial(_url, name, path_params, query_params, fragment)

        self.links = []

        if 3 < page:
            # 1
            self.links.append(
                (1, url(1), 1 == page)
            )

            # ...
            if 4 < page:
                self.links.append(
                    (False, '', False)
                )

        for i in range(1, pages_count + 1):
            if page - 3 < i < page + 3:
                self.links.append(
                    (i, url(i), i == page)
                )

        if page < pages_count - 2:
            # ...
            if page < pages_count - 3:
                self.links.append(
                    (False, '', False)
                )

            # count
            self.links.append(
                (pages_count, url(pages_count), pages_count == page)
            )

        self.iter = None
        self.slice = slice(
            (page - 1) * per_page,
            page * per_page
        )

    def __iter__(self):
        self.iter = iter(self.links)
        return self

    def __next__(self):
        return next(self.iter)

# private

def _url(name, path_params, query_params, fragment, page):
    path_params['page'] = page
    return '{}{}{}'.format(
        reverse(name, kwargs=path_params),
        '?{}'.format(urlencode(query_params)) if query_params else '',
        '#{}'.format(fragment) if fragment else ''
    )



# Generated by Django 2.1.7 on 2019-02-27 07:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_support_lite', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='ticket',
            name='priority',
            field=models.IntegerField(choices=[(0, 'Low'), (1, 'Normal'), (2, 'High')]),
        ),
    ]



from django.contrib.auth import get_user_model as user_model
from django.db import models

from django_support_lite.enums import Priority, ResponseType, Status


class TicketManager(models.Manager):
    def count_open(self, user=None):
        query = super().get_queryset().filter(status=Status.OPEN)

        if user:
            query = query.filter(user=user)

        return query.count()

    def count_pending(self):
        return super().get_queryset().filter(
            status=Status.OPEN,
            response_type=ResponseType.USER_RESPONSE
        ).count()

class Ticket(models.Model):
    id = models.AutoField(primary_key=True)
    user = models.ForeignKey(user_model(), related_name='user', on_delete=models.CASCADE)
    user_close = models.ForeignKey(user_model(), related_name='user_close', null=True, on_delete=models.CASCADE)
    title = models.CharField(max_length=300)
    response_type = models.IntegerField(choices=ResponseType.choices())
    status = models.IntegerField(choices=Status.choices())
    priority = models.IntegerField(choices=Priority.choices())
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    objects = TicketManager()



from setuptools import setup

setup(
    name='django-support-lite2',
    version='0.0.1',
    description='Django Support Lite 2',
    url='https://nii-ikt.ru/',
    author='nii-ikt',
    author_email='it-director@nii-ikt.ru',
    license='MIT',
    packages=[
        'django_support_lite',
        'django_support_lite.forms.fields',
        'django_support_lite.forms.inputs',
        'django_support_lite.forms.validators',
        'django_support_lite.helpers',
        'django_support_lite.migrations',
        'django_support_lite.models',
        'django_support_lite.templatetags',
        'django_support_lite.views',
        'django_support_lite.views.forms',
    ],
    include_package_data=True,
    zip_safe=False
)



from django import forms
from django.forms import ValidationError


class MultipleFileField(forms.FileField):
    def __init__(self, *args, **kwargs):
        self.min_count = kwargs.pop("min_count", None)
        self.max_count = kwargs.pop("max_count", None)

        super(MultipleFileField, self).__init__(*args, **kwargs)

    def clean(self, value, initial=None):
        files_count = len(value)

        if not self.required and files_count == 0:
            return []

        if self.min_count and files_count < self.min_count:
            raise ValidationError(
                "Необходимо указать не менее {} файлов, указано {}.".format(self.min_count, files_count)
            )

        if self.max_count and files_count > self.max_count:
            # TODO: i18n
            raise ValidationError(
                "Необходимо указать не более {} файлов, указано {}.".format(self.max_count, files_count)
            )

        for file in value:
            self.run_validators(file)

        return value
